/*    Copyright (C) 1999,  Daniel Schulz	   
 * 
 *    This file is part of the TDI library. The TDI library implements  
 *    the generic part of the Thread Debug Interface for POSIX Threads
 *    Implementations - TDI.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Report problems and direct all questions to:
 *
 *    tdi-bugs@informatik.hu-berlin.de
 *	
 * $Author: dschulz $ $Date: 1999/05/03 23:08:35 $ $Name: TDI-alpha-release-2 $ File: tdi-client.c
 *
 */
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>

#define TDI_REQUEST_TABLE_USE
#include "tdi.h"
#include "tdi-client.h"       
#include "tdi-comm.h"    /* abstract definition of communication */


/* ------------------------------------------------------ *
 *                decl, def stuff                         *
 * ------------------------------------------------------ */

/* TDI methods */
int TDI_init         (ThreadDebugInterfaceT* TDIPtr);
int TDI_openChannel  (ThreadDebugInterfaceT* TDIPtr);
int TDI_closeChannel (ThreadDebugInterfaceT* TDIPtr);
int TDI_send    (ThreadDebugInterfaceT* TDIPtr, char *request, int length);
int TDI_receive (ThreadDebugInterfaceT* TDIPtr, char **result, int *length);
int TDI_output  (ThreadDebugInterfaceT* TDIPtr, void* result, char* selection);
int TDI_error   (ThreadDebugInterfaceT* TDIPtr, char* result);

/* the TDI object */
ThreadDebugInterfaceT TDI = {

  /* which method implementation is used     */
  TDI_init,
  TDI_send, 
  TDI_receive,
  TDI_error,
  
  /* channel will be assigned later */
  (CommChannelT*) 0
};

/* the Comm Channel Object */
static CommChannelT TDI_Channel;

/* ------------------------------------------------------ *
 *                                                        *
 *                 basic TDI functions                    *
 *                                                        *
 * ------------------------------------------------------ */

int TDI_init(ThreadDebugInterfaceT* TDIPtr) {

  /* comm channel */
  TDIPtr->channel= &TDI_Channel;

  /* initialize the communication channel */
  newTDIChannel(TDIPtr->channel,SHARED_MEMORY_IPC);
}

int TDI_openChannel(ThreadDebugInterfaceT* TDIPtr) {
  int result;

  pid_t pid = getpid();

  
#if 0 /* dynamic loading of the TDI server will be realized next */
  /* we have to trigger the target to load the server */
  if (parse_and_eval_address("pthread_load_TDI_Server"
			      "(\"libTDISrv.so\",\"tdi_server\")")!=0) {
      printf_filtered("Error loading the TDI Server\n"); 
      return TDI_ERROR;
  }
#endif

  TDIPtr->channel->open(TDIPtr->channel,(void*)&pid);
  TDIPtr->channel->key = pid;
}

int TDI_closeChannel(ThreadDebugInterfaceT* TDIPtr) {
  TDIPtr->channel->close(TDIPtr->channel);
}


int TDI_send(ThreadDebugInterfaceT* TDIPtr, char *request, int length) {

  /* open the channel */

  TDI_openChannel(TDIPtr);

  /* send the data */

  if ( TDIPtr->channel->send(TDIPtr->channel,request,length) 
       == TDI_ERROR) 
    {
      /* close the channel */
      TDI_closeChannel(TDIPtr);
      /* signal an error */
      return TDI_ERROR;
    };

  return TDI_OK;
}

int TDI_receive(ThreadDebugInterfaceT* TDIPtr, char** result, int *length) {

  /* receive the data */

  if ( TDIPtr->channel->receive(TDIPtr->channel,result,length) 
       == TDI_ERROR)
    {
      /* close the channel */
      TDI_closeChannel(TDIPtr);
      /* signal an error */
      return TDI_ERROR;
    };

   /* close the channel */
   TDI_closeChannel(TDIPtr);
   
   return TDI_OK;

}


int 
TDI_error (TDIPtr,result) 
     ThreadDebugInterfaceT* TDIPtr; 
     char* result;
{
  if (!result)
    return 1;

  /* there is no empty result string */
  if (tolower(result[0])=='t') {
    /* thats the error symbol */
    return 1;
  }
  return 0;
}










