/*    Copyright (C) 1999,  Daniel Schulz	   
 * 
 *    This file is part of the TDI library. The TDI library implements  
 *    the generic part of the Thread Debug Interface for POSIX Threads
 *    Implementations - TDI.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Report problems and direct all questions to:
 *
 *    tdi-bugs@informatik.hu-berlin.de
 *	
 * $Author: dschulz $ $Date: 1999/05/03 23:08:59 $ $Name: TDI-alpha-release-2 $ File: ./Comm/tdi-comm.c
 *
 */
#include <stdio.h>

#include "tdi.h"
#include "tdi-comm.h"    /* abstract definition of communication */
#include "tdi-comm-sm.h" /* shared memory impl. of communication */


#ifdef TDI_SERVER
#include "smalloc.h"     /* secure malloc,calloc,realloc,free,strdup */
#include "tdi-dlgen.h"   /* secure standard functions */
#endif

int  TDIChannel_Init(struct CommChannelT* ChannelPtr);
int  TDIChannel_Open(struct CommChannelT* ChannelPtr, void* spec);
void TDIChannel_Close(struct CommChannelT* ChannelPtr);
int  TDIChannel_Send(struct CommChannelT* ChannelPtr, char  *data, int length);
int  TDIChannel_Receive (struct CommChannelT* ChannelPtr, 
			 char **data, int* length);

static CommChannelT TDICommChannelTemplate = {
  /* attributes */
  0,                /* key */
  UNDEF_IPC,    /* type */
  (IpcChannelT*) 0, /* Ipc Channel */
  
  /* methods */
  TDIChannel_Init,
  TDIChannel_Open,
  TDIChannel_Close,
  TDIChannel_Send,
  TDIChannel_Receive
}; 

static IpcChannelT IpcChannel;

void 
newTDIChannel(CommChannelT* channel, int IpcType) {
  
  /* FIXME: the IPC_Type should be read from a file to    *
   * be more flexible and configurable                    */

  *channel = TDICommChannelTemplate;

  /* assign the IPCType */
  channel->IpcChannel= &IpcChannel;
  channel->Type = IpcType;

  /* initialize the channel */
  channel->init(channel);
}

extern int newSMChannel();

int  
TDIChannel_Init(struct CommChannelT* ChannelPtr) {

  /* read the IPC Type and choose the methods */
  switch (ChannelPtr->Type) {
  case SHARED_MEMORY_IPC:
    newSMChannel(ChannelPtr->IpcChannel,(char *)0);
    break;
  default:
    return TDI_ERROR;
  }
  return TDI_OK;
}

int  
TDIChannel_Open(struct CommChannelT* ChannelPtr, void* spec) {

  return ChannelPtr->IpcChannel->open(ChannelPtr->IpcChannel,spec);

}

void 
TDIChannel_Close(struct CommChannelT* ChannelPtr) {
  ChannelPtr->IpcChannel->close(ChannelPtr->IpcChannel);
}

int   
TDIChannel_Send(struct CommChannelT* ChannelPtr, char  *data, int length) {

  /* send the frame */
  return ChannelPtr->IpcChannel->send(ChannelPtr->IpcChannel,data,length);

}

int  
TDIChannel_Receive (struct CommChannelT* ChannelPtr, char **data, int* length) {


  *data   = (char*) calloc(TDI_MAX_RESPONSE_BYTES,sizeof(char));

  ChannelPtr->IpcChannel->receive(ChannelPtr->IpcChannel,data,length);

  return TDI_OK;
}

























