/*    Copyright (C) 1999,  Daniel Schulz	   
 * 
 *    This file is part of the TDI library. The TDI library implements  
 *    the generic part of the Thread Debug Interface for POSIX Threads
 *    Implementations - TDI.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Report problems and direct all questions to:
 *
 *    tdi-bugs@informatik.hu-berlin.de
 *	
 * $Author: dschulz $ $Date: 1999/05/03 23:04:08 $ $Name: TDI-alpha-release-2 $ File: tdi-access-defs.h
 *
 */
#ifndef _tdi_access_defs_h_
#define _tdi_access_defs_h_

#define ATTRIBUTES_PER_QUERY 30
#define OPTIMIZATIONS_PER_QUERY 5

#define TDI_ACCESS_OK 0
#define TDI_ACCESS_ERROR 1

/* these are the attr string tables */
extern char *ThreadAccessTable[];
extern char *MutexAccessTable[];
extern char *CondAccessTable[];


/* bag of attributes */
typedef struct {
  RelTypeT RelType;
  int AttributeList[ATTRIBUTES_PER_QUERY]; /* list of projected attr */
#if ATTRIBUTES_PER_QUERY > 32
#error "ATTRIBUTES_PER_QUERY too high !"
#endif
  int SetMask;                             /* bit field for set attributes */
  int ValueList[ATTRIBUTES_PER_QUERY];     /* values to write */
  unsigned short NextEntry;                /* list counter */
  short SetDisplayMode;                    /* list or set output */
} TdiAccessListT;

/* bag of optimizations */

typedef enum {
  OT_UNKNOWN = -1,
  OT_SINGLE_THREAD = 0,
  OT_CURRENT_THREAD,
  /******/
  OT_COUNT
} OptimizationTypeT;

typedef struct {
  OptimizationTypeT OptimizationType;
  void* Specific;
} TdiOptimizationListElemT;

typedef struct {
  TdiOptimizationListElemT Optimization[OPTIMIZATIONS_PER_QUERY];
  unsigned short NextEntry;
} TdiOptimizationListT;


typedef struct {
  AttrDomainT (*get) (ObjRefT);
  AttrDomainT (*set) (ObjRefT, AttrDomainT); 
} PthreadImplAttrAccessT;  

typedef struct PthreadImplContainerAccess {
  void* (*getFirst)();             
  void* (*getNext )(ObjRefT Last);
} PthreadImplContainerAccessT; 


typedef struct PthreadImplEntityAccessT {
  /* getFirst, getNext */
  PthreadImplContainerAccessT EntityContainerAccess;
  /* all attribute functions */
  PthreadImplAttrAccessT*     EntityAttributeAccess;
} PthreadImplEntityAccessT;
 

/* global heros */
extern char* TdiAccessInputString; 
extern char* TdiAccessInputPtr;
extern char* TdiAccessInputLim;

/* error variables indicating an procomp or an rt_eval error */
extern int TDIPreCompError;
extern int TDI_RT_EvalError;

#endif




















