%{
  /*    Copyright (C) 1999,  Daniel Schulz	   
   * 
   *    This file is part of the TDI library. The TDI library implements  
   *    the generic part of the Thread Debug Interface for POSIX Threads
   *    Implementations - TDI.
   *
   *    This library is free software; you can redistribute it and/or
   *    modify it under the terms of the GNU Library General Public
   *    License as published by the Free Software Foundation; either
   *    version 2 of the License, or (at your option) any later version.
   *
   *    This library is distributed in the hope that it will be useful,
   *    but WITHOUT ANY WARRANTY; without even the implied warranty of
   *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   *    Library General Public License for more details.
   *
   *    You should have received a copy of the GNU Library General Public
   *    License along with this library; if not, write to the Free
   *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   *
   *    Report problems and direct all questions to:
   *
   *    tdi-bugs@informatik.hu-berlin.de
   *	
   * $Author: dschulz $ $Date: 1999/05/03 23:09:26 $ $Name: TDI-alpha-release-2 $ 
   *
   * File: tdi-access-y.y (TDI-QL parser spec)
   *
   */

#include <stdio.h>
#include "tdi.h"
#include "tdi-access.h"      /* used functions */
#include "tdi-access-defs.h" /* definitions */
 /* NOTE: malloc, calloc, realloc, free and strdup *
  *       are redefined                            */
#include "smalloc.h"

  /* standard functions are redefined */
#include "tdi-dlgen.h" 

#define pV(x) 

  /* this is the TDI version */
  char *TDIVersionString = "!$Name: TDI-alpha-release-2 $ $Date: 1999/05/03 23:09:26 $";

  typedef enum {
    TDI_EVAL_UNDEF_MODE=0,
    TDI_EVAL_PRECOMP_MODE, /* syntax check (per query) */
    TDI_EVAL_RTEVAL_MODE   /* evaluation   (per tupel) */
  } TdiEvalModeT;

  /* global variables */
  extern PthreadImplEntityAccessT* PthreadImplAccess;

  /* variables for the flex YY_INPUT macro */
  char* TdiAccessInputString; 
  char* TdiAccessInputPtr;
  char* TdiAccessInputLim;

  /* external function to flush the buffer */
  extern void my_yy_flush_buffer();

  /* generated info tables from the precompiler */
  TdiAccessListT* TdiAccessVisibleList=NULL;
  TdiAccessListT* TdiAccessComputedList=NULL;

  /* access optimization list */
  TdiOptimizationListT* TdiAccessOptiList = NULL;

  /* determined entity and container type */
  RelTypeT    RelType    = ET_UNDEF;

  /* the transformed access string */
  char* TdiPreCompResult;

  /* a boolean value telling whether the processed entity will *
   * be displayed or not                                       */
  int TdiAccessResult = 0;

  /* the type of currently processed entities */
  RelTypeT RTRelType = ET_UNDEF;
  
  /* the entity itself */
  void* RTCurrentEntity = NULL;
  
  /* a frequently used auxiliary string */
  static char TdiAccessTmpString[200];

  /* detect a state==RUNNING access */
  int TermExprIsCAttr = 0;
  int AttributeType   = -1;

  /* error variables indicating procomp or rt_eval errors */

  int TDIPreCompError  = 0;
  int TDI_RT_EvalError = 0;

  /* evaluation mode (precomp or rt_eval) */
  TdiEvalModeT TDIEvalMode = TDI_EVAL_UNDEF_MODE;

  /* set or list requested */
  int TdiSetMode = 0;

%}

%union {
  char* string;
  int   iconst;
  int  no_type;
}

%token VERSION TEDMODE TDIMODE AND OR LESS LESS_EQUAL GRT GRT_EQUAL NOTEQUAL EQUAL ASTERISK 
%token THREADREL MUTEXREL CONDREL HEX_CONST INT_CONST STRING_CONST 

%token <string> IDENT INIT 
%token <iconst> HEX_CONST INT_CONST
%type  <string> Attr VAttr CAttr TermExpr CmpExpr SpecExpr VList SpecExpression
%type  <iconst> Const 

%type  <iconst> RTTermExpr RTCmpExpr RTSpecExpr RTConst RTPlaceHolder


%left OR 
%left AND ','
%left LESS LESS_EQUAL GRT GRT_EQUAL EQUAL
%left '+' '-'
%left ASTERISK '/'

%%

Command: 
Query
| Init
| RuntimeEvaluation
;

Query: 	
  TEDMODE DistinctMode EntityAccess ':' 
  
  {

    /* Query was "<TED>..." */

    if ((TdiAccessVisibleList = 
      (TdiAccessListT*)calloc(1,sizeof(TdiAccessListT))) == NULL)
      yyerror();

    /* set or list */
    TdiAccessVisibleList->SetDisplayMode = TdiSetMode;

    if ((TdiAccessComputedList= 
      (TdiAccessListT*)calloc(1,sizeof(TdiAccessListT))) == NULL)
      yyerror();
    if ((TdiAccessOptiList= 
	 (TdiOptimizationListT*)
	 calloc(1,sizeof(TdiOptimizationListT))) == NULL)
      yyerror();

    TdiAccessVisibleList->RelType=RelType;
    TdiAccessComputedList->RelType=RelType;
    
  }

  SpecExpression Terminator 

  { 
    int i;
    char **AccessTable;
    /* print out the visible and computed list */
    pV(printf("\nVisible Attributes:\n"));


    /* select the keyword table */
    switch (RelType) {
    case ET_THREAD:
      AccessTable = ThreadAccessTable;
      break;
    case ET_MUTEX:
      AccessTable = MutexAccessTable;
      break;
    case ET_COND:
      AccessTable = CondAccessTable;
      break;
    default:
      yyerror();
    }

    /* for debug only */
    for (i=0;i<TdiAccessVisibleList->NextEntry;i++) {
      pV(printf("\t%s\n",AccessTable[TdiAccessVisibleList->AttributeList[i]]));
    }
    pV(printf("\nComputed Attributes:\n"));
    for (i=0;i<TdiAccessComputedList->NextEntry;i++) {
      pV(printf("\t%s\n",AccessTable[TdiAccessComputedList->AttributeList[i]]));
    }
    pV(printf("\n--accept--\n"));
    /* end of debug stuff */

    return TDI_ACCESS_OK;
  }
| TDIMODE VERSION Terminator 
  { 
    /* TDI version requested */
    TdiAccessVisibleList  = NULL; 
    TdiAccessComputedList = NULL;
    TdiAccessOptiList     = NULL;

    pV(printf("TDI mode\n"));

    TdiPreCompResult = strdup(TDIVersionString);
    
    pV(printf("%s\n",TdiPreCompResult));
    return TDI_ACCESS_OK;
  }
;


Init:
INIT Terminator {
  /* the init command is a good place to do some *
   * initial                                     */
  
  
  return TDI_ACCESS_OK;
}
   ;

DistinctMode:  '!' { 
                     pV(printf("All tupels, please!\n"));
		     TdiSetMode = 0;
                   }
                 | /* empty */ 
                 { 
		   pV(printf("A Set, please!\n"));
		   TdiSetMode = 1;
                 } 
               ;


EntityAccess: THREADREL   { RelType = ET_THREAD; pV(printf("Thread \n"));}
              | MUTEXREL  { RelType = ET_MUTEX; pV(printf("Mutex \n"));}
              | CONDREL   { RelType = ET_COND; pV(printf("Cond \n"));}
              ;

SpecExpression: VList ':' SpecExpr 
  { 
    pV(printf("\nResult: %s:%s\n",$1,$3)); 
    sprintf(TdiAccessTmpString,".%s\n",$3);
    TdiPreCompResult=(char*)strdup(TdiAccessTmpString);
    free($1);free($3);
 
  } 
  | VList 
  {
    pV(printf("\nResult: %s\n",$1));
    free($1);
  }
  ;

VList:
         VAttr ','  VList
	 {
	   sprintf(TdiAccessTmpString,"%s,%s",$1,$3);
	   free($1);free($3);
	   $$=(char*)strdup(TdiAccessTmpString);
	 }
         | VAttr {$$=$1;}
         ;

RuntimeEvaluation: '.' RTSpecExpr Terminator { TdiAccessResult = $2;return 0;};

RTSpecExpr:
          RTSpecExpr AND RTSpecExpr
	  {
	    pV(printf(" RTAND "));
	    $$ = ($1 && $3);
	  }
          | RTSpecExpr OR RTSpecExpr
	  {
	    pV(printf("%d RTOR %d",$1,$3));
	    $$ = ($1 || $3);
	  }
          | '(' RTSpecExpr ')' 
	  {
	    pV(printf(" () "));
	    $$ = $2;
	  }
          | RTCmpExpr 
          {
	    pV(printf(" RTComp "));
	    $$=$1;
	  } 
          ;

RTCmpExpr:
RTTermExpr LESS        RTConst 
{
  pV(printf(" RT< "));
  $$ = ($1 < $3);
}
| RTTermExpr LESS_EQUAL  RTConst
{
  pV(printf(" RT<= "));
  $$ = ($1 <= $3);
}
| RTTermExpr GRT         RTConst
{
  pV(printf(" RT> "));
  $$ = ($1 > $3);
}
| RTTermExpr GRT_EQUAL   RTConst
{
  pV(printf(" RT>= "));
  $$ = ($1 >= $3);
}
| RTTermExpr EQUAL       RTConst
{
  pV(printf("(%d RT== %d)",$1,$3));
  $$ = ($1 == $3);
}

| RTTermExpr NOTEQUAL       RTConst
{
  pV(printf(" RT!= "));
  $$ = ($1 != $3);
}
;

RTTermExpr:
RTPlaceHolder 
{
  pV(printf("PH(%d)",$1));
  /* $1 yields the number of the desired attribute */
  $$ = PthreadImplAccess[RTRelType].EntityAttributeAccess[$1].get(RTCurrentEntity);
}
| RTConst 
{ 
  pV(printf(" RTConst "));
  $$ = $1;
}
| RTTermExpr ASTERISK  RTTermExpr 
  {
    pV(printf(" RT* "));
    $$ = $1*$3;
  }
| RTTermExpr '/'  RTTermExpr      
  {
     pV(printf(" RT/ "));
    $$ = $1/$3;
   }
| RTTermExpr '-'  RTTermExpr      
  {
    pV(printf(" RT- "));
    $$ = $1-$3;
  }
| RTTermExpr '+'  RTTermExpr      
  {
    pV(printf(" RT+ "));
    $$ = $1+$3;
  }
;

RTPlaceHolder:
  '$' INT_CONST {  pV(printf(" Placeholder %d ",$2));$$=$2; }
;

RTConst:
  Const {  $$=$1;}
;

SpecExpr:
          SpecExpr AND SpecExpr
	  {
	    sprintf(TdiAccessTmpString,"%s&&%s",$1,$3);
	    free($1);free($3);
	    $$=(char*)strdup(TdiAccessTmpString);
	  }
          | SpecExpr OR SpecExpr
	  {
	    sprintf(TdiAccessTmpString,"%s||%s",$1,$3);
	    free($1);free($3);
	    $$=(char*)strdup(TdiAccessTmpString);
	  }
          | '(' SpecExpr ')' 
	  {
	    sprintf(TdiAccessTmpString,"(%s)",$2);
	    free($2);
	    $$=(char*)strdup(TdiAccessTmpString);
	  }
          | CmpExpr {$$=$1;} 
          ;

CmpExpr:
TermExpr LESS        Const 
{
  sprintf(TdiAccessTmpString,"%s<%d",$1,$3);
  free($1);
  $$=(char*)strdup(TdiAccessTmpString);

}
| TermExpr LESS_EQUAL  Const
{
  sprintf(TdiAccessTmpString,"%s<=%d",$1,$3);
  free($1);
  $$=(char*)strdup(TdiAccessTmpString);

}
| TermExpr GRT         Const
{
  sprintf(TdiAccessTmpString,"%s>%d",$1,$3);
  free($1);
  $$=(char*)strdup(TdiAccessTmpString);

}
| TermExpr GRT_EQUAL   Const
{
    sprintf(TdiAccessTmpString,"%s>=%d",$1,$3);
    free($1);
    $$=(char*)strdup(TdiAccessTmpString);

}
| TermExpr EQUAL       Const
{
  sprintf(TdiAccessTmpString,"%s==%d",$1,$3);

  /* detect a state==RUNNING access */
  if (TermExprIsCAttr && (RelType==ET_THREAD) && ($3==TDI_RUNNING_STATE) && (AttributeType == TA_STATE)) {
    /* we can add an optimzation hint to the optimization list */
    TdiAccessOptiList->Optimization[TdiAccessOptiList->NextEntry].
      OptimizationType = OT_CURRENT_THREAD;
    TdiAccessOptiList->NextEntry++;
  }

  free($1);
  $$=(char*)strdup(TdiAccessTmpString);

}
| TermExpr NOTEQUAL       Const
{
  sprintf(TdiAccessTmpString,"%s!=%d",$1,$3);
  free($1);
  $$=(char*)strdup(TdiAccessTmpString);
  
}  
;

TermExpr:
CAttr {$$=$1;pV(printf("}STA "));TermExprIsCAttr=1;}
| Const { sprintf(TdiAccessTmpString,"%d",$1);$$=(char*)strdup(TdiAccessTmpString);pV(printf("}STC "));TermExprIsCAttr=0;}
| TermExpr ASTERISK  TermExpr 
  {
    sprintf(TdiAccessTmpString,"%s*%s",$1,$3);
    free($1);free($3);
    $$=(char*)strdup(TdiAccessTmpString);
    pV(printf("}* "));
    TermExprIsCAttr=0;
  }
| TermExpr '/'  TermExpr      
  {
    sprintf(TdiAccessTmpString,"%s/%s",$1,$3);
    free($1);free($3);
    $$=(char*)strdup(TdiAccessTmpString);
    pV(printf("}/ "));
    TermExprIsCAttr=0;
  }
| TermExpr '-'  TermExpr      
  {
    sprintf(TdiAccessTmpString,"%s-%s",$1,$3);
    free($1);free($3);
    $$=(char*)strdup(TdiAccessTmpString);
    pV(printf("}- "));
    TermExprIsCAttr=0;
  }
| TermExpr '+'  TermExpr      
  {
    sprintf(TdiAccessTmpString,"%s+%s",$1,$3);
    free($1);free($3);
    $$=(char*)strdup(TdiAccessTmpString);
    pV(printf("}+ "));
    TermExprIsCAttr=0;
  }
;


CAttr:
  Attr 
  {
    AttributeType=-1;
    /* we add the Symbol to the specified list, if *
     * it doesn't contain it                       */
    if ((AttributeType=TdiAccessAddSymbol(RelType,$1,NULL,TdiAccessComputedList))
	==-1) {
      pV(printf("Attribute is wrong\n"));
      yyerror();
    }
    sprintf(TdiAccessTmpString,"$%d",AttributeType);
    free($1);
    $$=(char*)strdup(TdiAccessTmpString);
  }
;

VAttr: 
  Attr  '=' Const  
  {
    int pos;
    /* we add the Symbol to the specified list, if *
     * it doesn't contain it                       */
    if ((pos=TdiAccessAddSymbol(RelType,$1,&($3),TdiAccessVisibleList))
	==-1) {
      pV(printf("Attribute is wrong\n"));
      yyerror();
    }
    sprintf(TdiAccessTmpString,"#%d",pos);
    free($1);
    $$=(char*)strdup(TdiAccessTmpString);
  }
 | Attr 
 {
   int pos;
   /* we add the Symbol to the specified list, if *
    * it doesn't contain it                       */
   if ((pos=TdiAccessAddSymbol(RelType,$1,NULL,TdiAccessVisibleList))
       ==-1) {
     pV(printf("Attribute is wrong\n"));
     yyerror();
   }
   sprintf(TdiAccessTmpString,"#%d",pos);
   free($1);
   $$=(char*)strdup(TdiAccessTmpString);
 }

;

 

Attr:
  IDENT {}
;

Const:
  INT_CONST {$$=$1;pV(printf("ICons"));}
| HEX_CONST {$$=$1;pV(printf("HCons"));}
;

Terminator:
/* empty */
| '\n'
;



%%

#ifdef STANDALONE

void
main () {

  char* TestString = "aq.*: id,prio,state: id<10 || prio >= 10";
  
  TdiAccessInputString=TdiAccessInputPtr=TestString;
  TdiAccessInputLim = TdiAccessInputString+strlen(TdiAccessInputString);
  
  yyparse();
}

#endif

int 
TdiAccessPreCompile( /* in */
		    char*                     Request , 
		    /* out */ 
		    RelTypeT*              RelTypePtr , 
		    char**                    PreCompString,
		    TdiAccessListT**          VisibleList,
		    TdiAccessListT**          ComputedList,
		    TdiOptimizationListT**    OptiList) {

  /* flush the buffer */
  my_yy_flush_buffer();

  /* assign all global vars, they are used many times */

  TdiAccessInputString=TdiAccessInputPtr=Request;
  TdiAccessInputLim = TdiAccessInputString+strlen(TdiAccessInputString);
  
  TdiPreCompResult = NULL;
  TDIPreCompError  = 0;
  TDIEvalMode      = TDI_EVAL_PRECOMP_MODE;

  yyparse();

  /* the global TDIPreCompError indicates an error */

  if (TDIPreCompError)
    return TDI_ACCESS_ERROR;


  *RelTypePtr    = RelType;
  *PreCompString = TdiPreCompResult;
  *VisibleList   = TdiAccessVisibleList;
  *ComputedList  = TdiAccessComputedList;
  *OptiList      = TdiAccessOptiList;

  return TDI_ACCESS_OK;
}

int 
TdiAccessRTEvaluation(char* PreCompResult, 
		      RelTypeT RelType, 
		      void* EntityPtr) {

  /* flush the buffer */
  my_yy_flush_buffer();

  TdiAccessInputString=TdiAccessInputPtr=PreCompResult;
  TdiAccessInputLim = TdiAccessInputString+strlen(TdiAccessInputString);


  RTRelType = RelType;
  RTCurrentEntity = EntityPtr; 

  pV(printf("calling yyparse with \"%s\"\n",PreCompResult));

  TDI_RT_EvalError = 0;
  TDIEvalMode      = TDI_EVAL_RTEVAL_MODE;


  yyparse();

  /* the global TDIRTEvalError indicates an error */

  if (TDI_RT_EvalError)
    return 0; /* the return value will not be processed */ 
  
  return TdiAccessResult; 
}


/* yyerror implementation */

int 
yyerror(char* msg) {

  switch (TDIEvalMode) {
  case TDI_EVAL_PRECOMP_MODE: TDIPreCompError = 1; 
    break;
  case TDI_EVAL_RTEVAL_MODE : TDI_RT_EvalError = 1; 
    break;
  default: /* fatal error */
    fprintf(stderr,"fatal error in yyparse(): no eval_mode\n");
  }

  return 0;

}













