/*    Copyright (C) 1999,  Daniel Schulz	   
 * 
 *    This file is part of the TDI library. The TDI library implements  
 *    the generic part of the Thread Debug Interface for POSIX Threads
 *    Implementations - TDI.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Report problems and direct all questions to:
 *
 *    tdi-bugs@informatik.hu-berlin.de
 *	
 * $Author: dschulz $ $Date: 1999/05/03 23:04:08 $ $Name: TDI-alpha-release-2 $ File: tdi-access.c
 *
 */
#include <stdio.h>
#include <assert.h>

#define NDEBUG /* for assert */

#include "tdi.h"
#include "tdi-access-defs.h"
#include "tdi-access.h"
#include "tdi-comm.h"
#include "smalloc.h"
#include "tdi-dlgen.h"

#define pV(x)

extern PthreadImplEntityAccessT* PthreadImplAccess;
extern int CurrentThreadAccess;

extern void* pthread_self();

char* ThreadAccessTable[TA_COUNT] =
{
  /* don't change the order, it's according to *
   * Thread AttributeTypeT                     */
  "id",      
  "addr",
  "prio",
  "state",
  "rstate",
  "entry",
  "earg",
  "newpc",
  "sp",
  "mbo",
  "cvwf",
  "pid"
};

char* MutexAccessTable[MA_COUNT] =
{
  /* don't change the order, it's according to *
   * Mutex AttributeTypeT                     */
  "id",
  "addr",
  "owner"
};

char* CondAccessTable[] = {
  /* don't change the order, it's according to *
   * Cond AttributeTypeT                     */
  "id",
  "addr",
  "cmutex"
};


/* returns the position in the keyword table */

int
TdiAccessAddSymbol(RelTypeT RelType, char* Name, AttrDomainT *ValuePtr,TdiAccessListT* AccessList) 
{
  unsigned short it=0,end=0,AttrCode=0;
  char** AccessTable=(char**)0;
  
  /* wrong entity type, exit */
  if (AccessList->RelType != RelType)
    return -1;


  /* select the keyword table */
  
  switch(RelType) {
  case ET_THREAD:
    end = TA_COUNT;
    AccessTable=ThreadAccessTable;
    break;
  case ET_MUTEX:
    end = MA_COUNT;
    AccessTable=MutexAccessTable;
    break;
  case ET_COND:
    end = CA_COUNT;
    AccessTable=CondAccessTable;
    break;
  default:
    return TDI_ACCESS_ERROR;
  }
  
  /* if it is already in, we can return successfully */
  for (it=0;it < AccessList->NextEntry;it++) 
    {
      if (!strcmp(Name,AccessTable[AccessList->AttributeList[it]])) {
	if (ValuePtr) {
	  /* set the bit for that attr according to the position in *
	   * the visible list                                       */
	  AccessList->SetMask |= 0x1 << it; 
	  AccessList->ValueList[it] = *ValuePtr; /* assign the value */
	 
	  printf("Value %d assigned to list entry %d [mask=0x%x]\n",
		 *ValuePtr,it,
		 AccessList->SetMask);

	}
	return AccessList->AttributeList[it];
      }
    }
  
  /* if we are still in, the symbol can be added */

  for (AttrCode=0;AttrCode < end; AttrCode++) 
    {
      if (!strcmp(Name,AccessTable[AttrCode])) {
	/* known keyword -> insert *
	 * <AttrCode> is the keyword code  */
	AccessList->AttributeList[AccessList->NextEntry]=AttrCode;
	/* printf("\nreturn %d\n", AccessList->NextEntry); */
	if (ValuePtr) {
	  /* set the bit for that attr */
	  AccessList->SetMask |= 0x1 << AccessList->NextEntry;
	  /* assign the value */
	  AccessList->ValueList[AccessList->NextEntry] = *ValuePtr;
	  /* DEBUG message */
	  printf("Value %d assigned to list entry  %d [mask=0x%x]\n",
		 *ValuePtr,AccessList->NextEntry,AccessList->SetMask);

	}
	/* increment the member count */
	AccessList->NextEntry++;
	return AttrCode;
      }
    }

  /* the keyword was neither in nor legal */
  return -1;
}

/* ------------------------------------------------------- *
 *                   TDI main function                     *
 * +++++++++++++++++++++++++++++++++++++++++++++++++++++++ *
 * input:  request                                         *
 * output: response                                        *
 * ------------------------------------------------------- */

int
TdiAccessRequestHandler(char*                     request, 
			char**                    response) 
{

  RelTypeT RelType       = ET_UNDEF;
  char *PreCompResult          = NULL; /* can be freed after procesing */
  TdiAccessListT*  VisibleList = NULL; /* can be freed after procesing */
  TdiAccessListT*  ComputedList = NULL; /* can be freed after procesing */
  TdiOptimizationListT* OptiList = NULL;
  char requestplusnl[255];
  ObjRefT element = NULL;
  int vi=0,i;                        /* visible list iterator */
  int ProcessSelectionPredicate = 0; /* is there a predicate to process */ 

  /* FIXME: this has to be configurable */
  char *WholeResult = NULL; /* the whole result string */
  char ShortResult[3];      /* 3 bytes emergency string */
  int  length=0;           /* the length of the whole result */
  
  /* alloc and initialize it */

  if ((WholeResult = (char*) calloc(sizeof(char)* TDI_MAX_RESPONSE_BYTES,1))==NULL)
    goto Attribute_Not_Retrievable;

  WholeResult[0]=0; /* calloc does that too */

  sprintf(requestplusnl,"%s\n",request);

  /* parse the request string */
  
  TdiAccessPreCompile(requestplusnl, 
		      &RelType, 
		      &PreCompResult, 
		      &VisibleList,
		      &ComputedList,
		      &OptiList);
  
  /* if there was a parse error, we respond with an error string */

  if (TDIPreCompError)
    goto Attribute_Not_Retrievable;

  pV(printf("%sET: %d\n",PreCompResult,RelType));


  /* if there is a leading '!' in PreCompResult,  *
   * it was a administrative request "<TDI>..."   */

  if ((PreCompResult)?(*PreCompResult=='!'):0)
    {
      sprintf(&WholeResult[length],"%s#",&PreCompResult[1]);
      length += strlen(&WholeResult[length]);
      goto Attribute_Not_Retrievable;
    }
  
  /* TED request */

  /* if there is'nt sth. to display, we return */

  if (VisibleList == NULL) 
    {
      goto Attribute_Not_Retrievable;
    }
  
  /* if we can't compute the predicate, an error message is sent */ 

  for (i=0;i<ComputedList->NextEntry;i++) 
    {
      if (!PthreadImplAccess[RelType].
	  EntityAttributeAccess[ComputedList->AttributeList[i]].get)
	{
	  /* function not defined for desired attribute */
	  goto Attribute_Not_Retrievable;
	}
    }

  /* for x in Container */

  /* getFirst */
  element = PthreadImplAccess[RelType].EntityContainerAccess.
    getFirst();

  pV(printf("first elem: 0x%x\n",element));

  /* note: the following two loops are nearly the same, but for *
   * performance reasons the PreCompResult test was moved out   *
   * ( one test before the loop vs. one test per loop )         */

  /* Alternative 1: output with selection */
  if (PreCompResult) 
    ProcessSelectionPredicate = 1; 
  
  while (element) {
    
    /* if there is a predicate, we only select proper tupels, *
     * else all tupels are displayed                          */
    
#define CURR_TUPLE_SIZE 100

    if ((ProcessSelectionPredicate)?
	TdiAccessRTEvaluation(PreCompResult,RelType,element):
	1) {
      
      char CurrentTupel[CURR_TUPLE_SIZE]; /* every tupel is          *
					   * stored temporarily to   *
					   * provide set like output */
      int currIndex = 0;       /* current last position in * 
			       * that tupel */      

      CurrentTupel[0]=0; /* initialize it */

      /* projection:                     *
       * output according to VisibleList */ 
      for (vi=0;vi<VisibleList->NextEntry;vi++) {
	
	/* if there is a read function */
	if (PthreadImplAccess[RelType].EntityAttributeAccess
	    [VisibleList->AttributeList[vi]].get) 
	  {

	    /* print the result hexadecimal */
	    sprintf(&CurrentTupel[currIndex],
		    "%x ",
		    PthreadImplAccess[RelType].
		    EntityAttributeAccess
		    [VisibleList->AttributeList[vi]].get(element));
	    currIndex += strlen(&CurrentTupel[currIndex]);
	    	    
	  }
	else /* there is no read function */ 
	  { 
	    /* print a NULL */
	    sprintf(&CurrentTupel[currIndex],"? ");
	    currIndex += strlen(&CurrentTupel[currIndex]);
	  }
	
	/* was there a write access to this attribute */
	if ((VisibleList->SetMask & (0x1 << vi)) &&
	    PthreadImplAccess[RelType].EntityAttributeAccess
	    [VisibleList->AttributeList[vi]].set) 
	  {
	    PthreadImplAccess[RelType].EntityAttributeAccess
	      [VisibleList->AttributeList[vi]].
	      set(element,
		  VisibleList->ValueList[vi]);
	  }
	
	/* test this */
	assert(currIndex<=CURR_TUPLE_SIZE);

      }

      /* if we do set output, all duplicate tupels are removed */

      if (VisibleList->SetDisplayMode) {     /* result has set character */
	if  (strstr(WholeResult,CurrentTupel)==NULL) /* tupel is not in */
	  {
	    /* it's a new tupel or we are in list mode */
	    sprintf(&WholeResult[length],"%s#",CurrentTupel);
	    length += strlen(&WholeResult[length]);
	  }
      }
      else {
	sprintf(&WholeResult[length],"%s#",CurrentTupel);
	length += strlen(&WholeResult[length]);
      }
    }
    /* getNext */
    element=PthreadImplAccess[RelType].EntityContainerAccess.
      getNext(element);

    pV(printf("new elem: 0x%x\n",element));

  }
  
  /* we can free the precomputed result */
  if (PreCompResult)
    free(PreCompResult);
  
 Attribute_Not_Retrievable:
  
  /* avoid empty responses */
  if (length == 0) {
    WholeResult = ShortResult;
    sprintf(&WholeResult[length],"T#");
    length += strlen(&WholeResult[length]);
  }
  
  assert(length<=TDI_MAX_RESPONSE_BYTES);

  /* terminating it with 0 */
  WholeResult[(length)?length-1:0]='\0';
  // length--;

  /* assign the response */
  *response = (char*)strdup(&WholeResult[0]);
  
  pV(printf ("strlen:%d l:%d\n",strlen(&WholeResult[0]),length));
  pV(printf ("WholeResult:\n%s\n",&WholeResult[0]));

  if (VisibleList)
    free (VisibleList);
  if (ComputedList)
    free (ComputedList);
  if (OptiList)
    free (OptiList); 
  if (WholeResult && length > 2)
    free(WholeResult);


  return length;
}



















