%{
  /*    Copyright (C) 1999,  Daniel Schulz	   
   * 
   *    This file is part of the TDI library. The TDI library implements  
   *    the generic part of the Thread Debug Interface for POSIX Threads
   *    Implementations - TDI.
   *
   *    This library is free software; you can redistribute it and/or
   *    modify it under the terms of the GNU Library General Public
   *    License as published by the Free Software Foundation; either
   *    version 2 of the License, or (at your option) any later version.
   *
   *    This library is distributed in the hope that it will be useful,
   *    but WITHOUT ANY WARRANTY; without even the implied warranty of
   *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   *    Library General Public License for more details.
   *
   *    You should have received a copy of the GNU Library General Public
   *    License along with this library; if not, write to the Free
   *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   *
   *    Report problems and direct all questions to:
   *
   *    tdi-bugs@informatik.hu-berlin.de
   *	
   * $Author: dschulz $ $Date: 1999/05/03 23:04:08 $ $Name: TDI-alpha-release-2 $ 
   *
   * File: tdi-access.lex (TDI-QL lexer)
   *
   */


#include "tdi-access-y.h"
  /* NOTE: malloc, calloc, realloc, free and strdup *
   *       are redefined                            */
#include "smalloc.h"
#include "tdi-dlgen.h"
#undef isatty

  /* --------------------------------------------- *
   * Description: lexer for the tdi access parser  *
   * Author:      Daniel Schulz                    *
   * Creation Date: 98-08-15                       *
   * Last Modified:                                *
   * --------------------------------------------- */
  
  
  /* --------------------------------------------- *
   * - we use keywords for entity access           *
   * - attributes are read in as idents            *
   * - constants are used and operators            *
   * --------------------------------------------- */

int yylineno = 0;
extern void my_yy_flush_buffer();
extern int my_yyinput(char*,int);

#undef YY_INPUT
#define YY_INPUT(b,r,ms) {r=my_yyinput(b,ms);}
#define MIN(a,b) (((a)<(b))?(a):(b))

#undef LEX_VERBOSE 
//#define LEX_VERBOSE 

#ifdef LEX_VERBOSE
#define V(x) x  
#else
#define V(x)
#endif

%}
IDENT [a-zA-Z]+
INT_CONST ([1-9][0-9]*|0)
HEX_CONST 0x[0-9a-fA-F]+
LOG_AND "&&"
LOG_OR  "||"
LESS    "<"
LESS_EQUAL "<="
GRT     ">"
GRT_EQUAL ">="
EQUAL   "=="
ASTERISK "*"
NOTEQUAL "!="
TEDMODE  "<TED>"
TDIMODE  "<TDI>"
%%

{HEX_CONST} {V(printf("HEX_CONST"));sscanf(&yytext[2],"%x",&yylval.iconst); return HEX_CONST;}
{INT_CONST} {V(printf("INT_CONST"));yylval.iconst = atoi(&yytext[0]); return INT_CONST;}
{LOG_AND} { V(printf("LOG_AND")); return AND;}
{LOG_OR}  { V(printf("LOG_OR")); return OR;}
{TEDMODE} { V(printf("TEDMODE")); return TEDMODE;}
{TDIMODE} { V(printf("TDIMODE")); return TDIMODE;}
{LESS}    { V(printf("LESS")); return LESS;}
{LESS_EQUAL} {V(printf("LESS_EQUAL")); return LESS_EQUAL;}
{GRT}        {V(printf("GRT")); return GRT;}
{GRT_EQUAL}  {V(printf("GRT_EQUAL")); return GRT_EQUAL;}
{EQUAL}      {V(printf("EQUAL")); return EQUAL;}
{NOTEQUAL}    {V(printf("NOTEQUAL")); return NOTEQUAL;}
Init|INIT|init         {V(printf("INIT")); return INIT;}
Version|version|VERSION {V(printf("version")); return VERSION;}
MUTEX|mutex       {V(printf("MUTEX")); return MUTEXREL;}
COND|cond         {V(printf("COND")); return CONDREL;}
THREAD|thread     {V(printf("THREAD")); return THREADREL;}
{ASTERISK}   {V(printf("ASTERISK")); return ASTERISK;}
{IDENT} {V(printf("IDENT")); yylval.string = (char*)strdup(&yytext[0]);return IDENT;}
[ \t] {V(printf("WS"));}
\n { V(printf("%c",yytext[0]));yylineno++;return yytext[0];}
.  { V(printf("[.]%c",yytext[0]));return yytext[0]; }
%% 

int 
yywrap() {
  return 1;
}


#ifndef STANDALONE 

extern char* TdiAccessInputString,*TdiAccessInputPtr,*TdiAccessInputLim;



int 
my_yyinput(char* buf, int max_size) {

  int n;

  n=MIN(max_size,TdiAccessInputLim-TdiAccessInputPtr);

  if (n>0) {
    memcpy(buf,TdiAccessInputPtr,n);
    TdiAccessInputPtr += n;
  }

  return n;
}

void
my_yy_flush_buffer() {
  YY_FLUSH_BUFFER;
}

#endif
