/*    Copyright (C) 1999,  Daniel Schulz	   
 * 
 *    This file is part of the TDI library. The TDI library implements  
 *    the generic part of the Thread Debug Interface for POSIX Threads
 *    Implementations - TDI.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Report problems and direct all questions to:
 *
 *    tdi-bugs@informatik.hu-berlin.de
 *	
 * $Author: dschulz $ $Date: 1999/05/03 23:09:40 $ $Name: TDI-alpha-release-2 $ File: tdi-comm-sm.h
 *
 */
#ifndef __sm_comm_chan_h__
#define __sm_comm_chan_h__
/* ------------------------------------------------- *
 * Description   : implementation of a shared memory *
 *                 based communication channel       *
 * Author        : Daniel Schulz                     * 
 * Creation Date : 98-06-18                          *
 * Last modified :                                   *
 * ------------------------------------------------- */


/* defines */

/* all can read and write the shared memory */
#define SHM_PERMS 0666

/* Bytes to code the length */
#define SM_HEADER_BYTES (sizeof(sm_page_header_t)) 

/* the length of a shared memory packet (data) */
#define SM_PAGE_SIZE (TDI_COMM_MAX_PACKET_SIZE - SM_HEADER_BYTES)

/* this defines the number of attempts to read the *
 * shared memory after sending the request         */
#define TDI_SM_MAX_ATTEMPT_TO_RECEIVE_DATA 100000

/* special sm specific data */

typedef struct {
  int ReadyToWrite;           /* the Lock           */
  int NewData;                /* for framing        */
  int synchronized;           /* enables locking    */
  int SMPageSize;             /* size of SM segment */
} sm_u_data;

/* a header for every SM page */

typedef struct {
  int shore;              /* the id of the sender      */
  int length;             /* packet length             */
  unsigned char follows;  /* a packet follows          */
} sm_page_header_t;

/* prototypes for the CommChannel methods */

int  sm_open    (IpcChannelT*, void*);
int  sm_send    (IpcChannelT*, char  *data, int length);
int  sm_receive (IpcChannelT*, char **data, int *length);
void sm_close   (IpcChannelT*);

/* synchronisation stuff */

#define SEM_PERMS 0666
#define SEM_CNT 1

#define SEM_FULL  1
#define SEM_EMPTY 0


int  SemOpen (int,int);
void SemClose(int);
int  SemGive (int);
int  SemGet  (int);
#endif



























